/*
 * @(#)JTextAttributesDialog.java	Version 1.0  <April 02, 2003>
 *
 * Copyright 2000-2002 Elegant MicroWeb Technologies Pvt. Ltd. (EMTPL). All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * - Redistributions of source code is not permitted under any conditions.
 * 
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of Elegant MicroWeb Technologies Pvt. Ltd. or the names of contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. EMTPL AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES OR LIABILITIES
 * SUFFERED BY LICENSEE AS A RESULT OF  OR RELATING TO USE, MODIFICATION
 * OR DISTRIBUTION OF THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL
 * EMTPL OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE
 * DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,
 * ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF EMTPL HAS
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 * You acknowledge that Software is not designed, licensed or intended
 * for use in the design, construction, operation or maintenance of any
 * nuclear facility or any other illegal activity.
 */

package common;

import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.beans.*;

/**
*	This utility class propvides the GUI for setting text attributes (
*	background, foreground, alignment, first line indent, left indent, right indent, line spacing,
*	space above, space below, 
*	font name, font size, bold, italic, underline, strike through, superscript, subscript ).
*/
public class JTextAttributesDialog extends JDialog implements ActionListener
{
	private JButton btBackground, btForeground,
										btOk, btCancel, btApply;
	private JComboBox cbAlignment, cbLineSpacing, cbSpaceAbove, cbSpaceBelow, 
											cbFirstLineIndent, cbLeftIndent, cbRightIndent, 
												cbFontFamily, cbFontSize;
	private JLabel labelAlignment, labelLineSpacing, labelSpaceAbove, labelSpaceBelow, 
											labelFirstLineIndent, labelLeftIndent, labelRightIndent, 
												labelFontFamily, labelFontSize;
	private JCheckBox ckBold, ckItalic, ckUnderline, 
											ckStrikeThrough, ckSubscript, ckSuperscript;
	private JTextPane invoker, textPreview;
	private PropertyEditor colorEditor;
	
	public JTextAttributesDialog(JTextPane invoker)
	{
		this(invoker,(Frame)null);
	}
	public JTextAttributesDialog(JTextPane invoker, Dialog owner)
	{
		super(owner,"TextAttributes",true);
		this.invoker = invoker;
		init();
	}
	public JTextAttributesDialog(JTextPane invoker, Frame owner)
	{
		super(owner,"TextAttributes",true);
		this.invoker = invoker;
		init();
	}
	private void init()
	{
		colorEditor = PropertyEditorManager.findEditor(java.awt.Color.class);
		
		initComponents();
		addComponents();
		initContents();
		initAttributes();
		addListeners();
		
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setResizable(false);
		
		show();
	}
	private void initComponents()
	{
		btOk = new JButton("OK");
		btOk.setMnemonic('K');
		
		btCancel = new JButton("Cancel");
		btCancel.setMnemonic('C');
		
		btApply = new JButton("Apply");
		btApply.setMnemonic('A');
		
		btBackground = new JButton("Background");
		btBackground.setMnemonic('B');
		btBackground.setBorder(null);
		btBackground.setForeground(getInvertColor(btBackground.getBackground()));
		
		btForeground = new JButton("Foreground");
		btForeground.setMnemonic('F');
		btForeground.setBorder(null);
		btForeground.setForeground(getInvertColor(btBackground.getBackground()));
		
		cbAlignment = new JComboBox();
		
		cbLineSpacing = new JComboBox();
		cbLineSpacing.setEditable(true);
		
		cbSpaceAbove = new JComboBox();
		cbSpaceAbove.setEditable(true);
		
		cbSpaceBelow = new JComboBox();
		cbSpaceBelow.setEditable(true);
		
		cbFirstLineIndent = new JComboBox();
		cbFirstLineIndent.setEditable(true);
		
		cbLeftIndent = new JComboBox();
		cbLeftIndent.setEditable(true);
		
		cbRightIndent = new JComboBox();
		cbRightIndent.setEditable(true);
		
		cbFontFamily = new JComboBox();
		
		cbFontSize = new JComboBox();
		cbFontSize.setEditable(true);		
		
		labelAlignment = new JLabel("  Alignment");
		labelAlignment.setForeground(Color.black);
		labelAlignment.setDisplayedMnemonic('G');
		labelAlignment.setLabelFor(cbAlignment);
		
		labelFirstLineIndent = new JLabel("  FirstLineIndent");
		labelFirstLineIndent.setForeground(Color.black);
		labelFirstLineIndent.setDisplayedMnemonic('I');
		labelFirstLineIndent.setLabelFor(cbFirstLineIndent);
		
		labelLeftIndent = new JLabel("  LeftIndent");
		labelLeftIndent.setForeground(Color.black);
		labelLeftIndent.setDisplayedMnemonic('L');
		labelLeftIndent.setLabelFor(cbLeftIndent);
		
		labelRightIndent = new JLabel("  RightIndent");
		labelRightIndent.setForeground(Color.black);
		labelRightIndent.setDisplayedMnemonic('R');
		labelRightIndent.setLabelFor(cbRightIndent);

		labelLineSpacing = new JLabel("  LineSpacing");
		labelLineSpacing.setForeground(Color.black);
		labelLineSpacing.setDisplayedMnemonic('S');
		labelLineSpacing.setLabelFor(cbLineSpacing);
		
		labelSpaceAbove = new JLabel("  SpaceAbove");
		labelSpaceAbove.setForeground(Color.black);
		labelSpaceAbove.setDisplayedMnemonic('E');
		labelSpaceAbove.setLabelFor(cbSpaceAbove);
		
		labelSpaceBelow = new JLabel("  SpaceBelow");
		labelSpaceBelow.setForeground(Color.black);
		labelSpaceBelow.setDisplayedMnemonic('W');
		labelSpaceBelow.setLabelFor(cbSpaceBelow);

		labelFontFamily = new JLabel("  Name");
		labelFontFamily.setForeground(Color.black);
		labelFontFamily.setDisplayedMnemonic('N');
		labelFontFamily.setLabelFor(cbFontFamily);
		
		labelFontSize = new JLabel("  Size");
		labelFontSize.setForeground(Color.black);
		labelFontSize.setDisplayedMnemonic('Z');
		labelFontSize.setLabelFor(cbFontSize);
		
		ckBold = new JCheckBox("  Bold");
		ckBold.setMnemonic('O');
		
		ckItalic = new JCheckBox("  Italic");
		ckItalic.setMnemonic('T');
		
		ckUnderline = new JCheckBox("  Underline");
		ckUnderline.setMnemonic('D');
		
		ckStrikeThrough = new JCheckBox("  StrikeThrough");
		ckStrikeThrough.setMnemonic('H');
		
		ckSubscript = new JCheckBox("  Subscript");
		ckSubscript.setMnemonic('U');
		
		ckSuperscript = new JCheckBox("  Superscript");
		ckSuperscript.setMnemonic('P');
		
	 	textPreview = new JTextPane();
	 	textPreview.setOpaque(false);
	}
	private void addComponents()
	{
		JPanel alignmentPanel = new JPanel();
		alignmentPanel.setLayout(new GridLayout(7,2,5,5));
		alignmentPanel.setBorder(new TitledBorder(
																new LineBorder(Color.gray), 
																"Indentation",
																TitledBorder.CENTER,
																TitledBorder.CENTER,
																getFont(),
																Color.gray
															));
		
		alignmentPanel.add(labelAlignment);
		alignmentPanel.add(cbAlignment);
		alignmentPanel.add(labelFirstLineIndent);
		alignmentPanel.add(cbFirstLineIndent);
		alignmentPanel.add(labelLeftIndent);
		alignmentPanel.add(cbLeftIndent);
		alignmentPanel.add(labelRightIndent);
		alignmentPanel.add(cbRightIndent);
		alignmentPanel.add(labelLineSpacing);
		alignmentPanel.add(cbLineSpacing);
		alignmentPanel.add(labelSpaceAbove);
		alignmentPanel.add(cbSpaceAbove);
		alignmentPanel.add(labelSpaceBelow);
		alignmentPanel.add(cbSpaceBelow);
		
		JPanel fontPanel = new JPanel();
		fontPanel.setLayout(new GridLayout(8,2,5,5));
		fontPanel.setBorder(new TitledBorder(
																new LineBorder(Color.gray), 
																"Font",
																TitledBorder.CENTER,
																TitledBorder.CENTER,
																getFont(),
																Color.gray
															));
		
		fontPanel.add(labelFontFamily);
		fontPanel.add(cbFontFamily);
		fontPanel.add(labelFontSize);
		fontPanel.add(cbFontSize);
		fontPanel.add(ckBold);
		fontPanel.add(ckStrikeThrough);
		fontPanel.add(ckItalic);
		fontPanel.add(ckSubscript);
		fontPanel.add(ckUnderline);
		fontPanel.add(ckSuperscript);
		
		JPanel colorPanel = new JPanel();
		colorPanel.setLayout(new GridLayout(1,2,5,5));
		colorPanel.setBorder(new TitledBorder(
																new LineBorder(Color.gray), 
																"Color",
																TitledBorder.CENTER,
																TitledBorder.CENTER,
																getFont(),
																Color.gray
															));
		
		colorPanel.add(btBackground);
		colorPanel.add(btForeground);
		
		JPanel previewPanel = new JPanel();
		previewPanel.setLayout(new BorderLayout(5,5));
		previewPanel.setBorder(new TitledBorder(
																new LineBorder(Color.gray), 
																"Preview",
																TitledBorder.CENTER,
																TitledBorder.CENTER,
																getFont(),
																Color.gray
															));
		JScrollPane scrollPane = new JScrollPane(textPreview);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setPreferredSize(new Dimension(200,70));
		previewPanel.add(scrollPane);
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT,5,5));
		
		buttonPanel.add(btOk);
		buttonPanel.add(btApply);
		buttonPanel.add(btCancel);
		
		JPanel mainPanel = new JPanel();
		GridBagLayout gbl = new GridBagLayout();
		mainPanel.setLayout(gbl);

   	Object[] gridBagItems = 
   	{
   		alignmentPanel,fontPanel,colorPanel,
   		previewPanel,buttonPanel
   	};

	  int gridx[] 			= {0,1,0,0,0};
	  int gridy[] 			= {0,0,1,2,3};
	  int gridwidth[] 	= {1,1,2,2,2};
	  int gridheight[] 	= {1,1,1,1,1};
	
	  GridBagConstraints gridBagConstraints[] = new GridBagConstraints[6];
	  for(int i=0;i<5;++i)
	  {
	   	gridBagConstraints[i] = new GridBagConstraints();
	   	gridBagConstraints[i].fill=GridBagConstraints.BOTH;
	   	gridBagConstraints[i].gridx=gridx[i];
	   	gridBagConstraints[i].gridy=gridy[i];
	   	gridBagConstraints[i].gridwidth=gridwidth[i];
	   	gridBagConstraints[i].gridheight=gridheight[i];
	   	gridBagConstraints[i].insets=new Insets(2,2,2,2);
	   	gbl.setConstraints((java.awt.Component)gridBagItems[i],gridBagConstraints[i]);
	   	mainPanel.add((java.awt.Component)gridBagItems[i]);
	  }

		getContentPane().setLayout(new BorderLayout(5,5));
		getContentPane().add(mainPanel);
		
		Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		
		setBounds((screenSize.width - (getPreferredSize().width + 100))/2, (screenSize.height - (getPreferredSize().height + 100))/2, getPreferredSize().width + 100, getPreferredSize().height + 100);
	}
	private void initContents()
	{
		String[] sAlign = { "Justified", "Left", "Center", "Right" };
		
		for(int i=0; i<sAlign.length; i++)
			cbAlignment.addItem(sAlign[i]);
		
		for(float i=-5.0f; i<6.0f; i++)
			cbFirstLineIndent.addItem(String.valueOf(i));
			
		for(float i=-5.0f; i<6.0f; i++)
			cbLeftIndent.addItem(String.valueOf(i));

		for(float i=-5.0f; i<6.0f; i++)
			cbRightIndent.addItem(String.valueOf(i));

		for(float i=-5.0f; i<6.0f; i++)
			cbLineSpacing.addItem(String.valueOf(i));

		for(float i=-5.0f; i<6.0f; i++)
			cbSpaceAbove.addItem(String.valueOf(i));

		for(float i=-5.0f; i<6.0f; i++)
			cbSpaceBelow.addItem(String.valueOf(i));

		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		String[] sFonts = ge.getAvailableFontFamilyNames();

		for(int i=0; i<sFonts.length; i++)
			cbFontFamily.addItem(sFonts[i]);

		int[] iSize = { 3, 5, 8, 10, 12, 14, 18, 24, 36, 48 };

		for(int i=0; i<iSize.length; i++)
			cbFontSize.addItem(String.valueOf(iSize[i]));
		
		String text = "This is a sample text.\n"+
										"You can set the text attributes.\n"+
										"This only works if the selected text is not null in the JTextPane.";	
		textPreview.setText(text);
	}
	private void initAttributes()
	{
		SimpleAttributeSet characterAttributes = new SimpleAttributeSet(invoker.getCharacterAttributes());
		SimpleAttributeSet paragraphAttributes = new SimpleAttributeSet(invoker.getParagraphAttributes());

		int iAlignment = StyleConstants.getAlignment(paragraphAttributes);
		switch(iAlignment)
		{
			case StyleConstants.ALIGN_JUSTIFIED :
				cbAlignment.setSelectedItem("Justified");
				break;
			case StyleConstants.ALIGN_LEFT :
				cbAlignment.setSelectedItem("Left");
				break;
			case StyleConstants.ALIGN_CENTER :
				cbAlignment.setSelectedItem("Center");
				break;
			case StyleConstants.ALIGN_RIGHT :
				cbAlignment.setSelectedItem("Right");
				break;
		}
		cbFirstLineIndent.setSelectedItem(String.valueOf(StyleConstants.getFirstLineIndent(paragraphAttributes)));
		cbLeftIndent.setSelectedItem(String.valueOf(StyleConstants.getLeftIndent(paragraphAttributes)));
		cbRightIndent.setSelectedItem(String.valueOf(StyleConstants.getRightIndent(paragraphAttributes)));
		cbLineSpacing.setSelectedItem(String.valueOf(StyleConstants.getLineSpacing(paragraphAttributes)));
		cbSpaceAbove.setSelectedItem(String.valueOf(StyleConstants.getSpaceAbove(paragraphAttributes)));
		cbSpaceBelow.setSelectedItem(String.valueOf(StyleConstants.getSpaceBelow(paragraphAttributes)));
		cbFontFamily.setSelectedItem(StyleConstants.getFontFamily(characterAttributes));
		cbFontSize.setSelectedItem(String.valueOf(StyleConstants.getFontSize(characterAttributes)));
		ckBold.setSelected(StyleConstants.isBold(characterAttributes));
		ckItalic.setSelected(StyleConstants.isItalic(characterAttributes));
		ckUnderline.setSelected(StyleConstants.isUnderline(characterAttributes));
		ckStrikeThrough.setSelected(StyleConstants.isStrikeThrough(characterAttributes));
		ckSubscript.setSelected(StyleConstants.isSubscript(characterAttributes));
		ckSuperscript.setSelected(StyleConstants.isSuperscript(characterAttributes));
		btBackground.setBackground(StyleConstants.getBackground(characterAttributes));
		btForeground.setBackground(StyleConstants.getForeground(characterAttributes));
		
		if(btBackground.getBackground().equals(Color.black))
			btBackground.setBackground(Color.white);	
		
		btBackground.setForeground(getInvertColor(btBackground.getBackground()));
		btForeground.setForeground(getInvertColor(btForeground.getBackground()));

		preview(false,false);
	}
	private void addListeners()
	{
		btBackground.addActionListener(this);
		btForeground.addActionListener(this);
		btOk.addActionListener(this);
		btCancel.addActionListener(this);
		btApply.addActionListener(this);

		cbAlignment.addActionListener(this);
		cbLineSpacing.addActionListener(this);
		cbSpaceAbove.addActionListener(this);
		cbSpaceBelow.addActionListener(this);
		cbFirstLineIndent.addActionListener(this);
		cbLeftIndent.addActionListener(this);
		cbRightIndent.addActionListener(this);
		cbFontFamily.addActionListener(this);
		cbFontSize.addActionListener(this);
		
		ckBold.addActionListener(this);
		ckItalic.addActionListener(this);
		ckUnderline.addActionListener(this);
		ckStrikeThrough.addActionListener(this);
		ckSubscript.addActionListener(this);
		ckSuperscript.addActionListener(this);
	}
	public void actionPerformed(ActionEvent e)
	{
		boolean isSet = false, isDispose = false;
		Object source = e.getSource();
		
		if(source == btBackground)
		{
			colorEditor.setValue(btBackground.getBackground());
			new ExtJDialog(getFrame(), "Color Editor", colorEditor.getCustomEditor());
			btBackground.setBackground((Color)colorEditor.getValue());
			btBackground.setForeground(getInvertColor(btBackground.getBackground()));
		}
		else if(source == btForeground)
		{
			colorEditor.setValue(btForeground.getBackground());
			new ExtJDialog(getFrame(), "Color Editor", colorEditor.getCustomEditor());
			btForeground.setBackground((Color)colorEditor.getValue());
			btForeground.setForeground(getInvertColor(btForeground.getBackground()));
		}
		else if(source == btOk)
		{
			isSet = true;
			isDispose = true;
		}
		else if(source == btCancel)
		{
			dispose();
		}
		else if(source == btApply)
		{
			isSet = true;
		}
		preview(isSet,isDispose);
	}
	private void preview(boolean isSet, boolean isDispose)
	{
		try
		{
			SimpleAttributeSet characterAttributes = getCharacterAttributes();
			SimpleAttributeSet paragraphAttributes = getParagraphAttributes();
			
	 		textPreview.setEditable(true);
	 		textPreview.selectAll();
			textPreview.setCharacterAttributes(characterAttributes,false);
			textPreview.setParagraphAttributes(paragraphAttributes,false);
			textPreview.setEditable(false);

			if(isSet)
			{
				invoker.setCharacterAttributes(characterAttributes,false);
				invoker.setParagraphAttributes(paragraphAttributes,false);
			}
			if(isDispose)
				dispose();
		}
		catch(Exception e)
		{
			// ignore
		}
	}
	private SimpleAttributeSet getCharacterAttributes()
	{
		SimpleAttributeSet attributes = new SimpleAttributeSet(invoker.getCharacterAttributes());

		int iFontSize;
		String sFontFamily;
		boolean bBold, bItalic, bUnderline, bStrikeThrough, bSubscript, bSuperscript;
		Color colorBack, colorFore;

		sFontFamily = cbFontFamily.getSelectedItem().toString();
		iFontSize = Integer.parseInt(cbFontSize.getSelectedItem().toString());
		bBold = ckBold.isSelected();
		bItalic = ckItalic.isSelected();
		bUnderline = ckUnderline.isSelected();
		bStrikeThrough = ckStrikeThrough.isSelected();
		bSubscript = ckSubscript.isSelected();
		bSuperscript = ckSuperscript.isSelected();
		colorBack = btBackground.getBackground();
		colorFore = btForeground.getBackground();

		StyleConstants.setFontFamily(attributes,sFontFamily);
		StyleConstants.setFontSize(attributes,iFontSize);
		StyleConstants.setBold(attributes,bBold);
		StyleConstants.setItalic(attributes,bItalic);
		StyleConstants.setUnderline(attributes,bUnderline);
		StyleConstants.setStrikeThrough(attributes,bStrikeThrough);
		StyleConstants.setSubscript(attributes,bSubscript);
		StyleConstants.setSuperscript(attributes,bSuperscript);
		StyleConstants.setBackground(attributes,colorBack);
		StyleConstants.setForeground(attributes,colorFore);

		return attributes;
	}
	private SimpleAttributeSet getParagraphAttributes()
	{
		SimpleAttributeSet attributes = new SimpleAttributeSet(invoker.getParagraphAttributes());

		int iAlignment;
		float fFirstLineIndent, fLeftIndent, fRightIndent, fLineSpacing,fSpaceAbove, fSpaceBelow;

		int index = cbAlignment.getSelectedIndex();
		switch(index)
		{
			case 0 :
				iAlignment = StyleConstants.ALIGN_JUSTIFIED;
				break;
			case 1 :
				iAlignment = StyleConstants.ALIGN_LEFT;
				break;
			case 2 :
				iAlignment = StyleConstants.ALIGN_CENTER;
				break;
			case 3 :
				iAlignment = StyleConstants.ALIGN_RIGHT;
				break;
			default :
				iAlignment = -1;
		}
		fFirstLineIndent = Float.parseFloat(cbFirstLineIndent.getSelectedItem().toString());
		fLeftIndent = Float.parseFloat(cbLeftIndent.getSelectedItem().toString());
		fRightIndent = Float.parseFloat(cbRightIndent.getSelectedItem().toString());
		fLineSpacing = Float.parseFloat(cbLineSpacing.getSelectedItem().toString());
		fSpaceAbove = Float.parseFloat(cbSpaceAbove.getSelectedItem().toString());
		fSpaceBelow = Float.parseFloat(cbSpaceBelow.getSelectedItem().toString());

		StyleConstants.setAlignment(attributes,iAlignment);
		StyleConstants.setFirstLineIndent(attributes,fFirstLineIndent);
		StyleConstants.setLeftIndent(attributes,fLeftIndent);
		StyleConstants.setRightIndent(attributes,fRightIndent);
		StyleConstants.setLineSpacing(attributes,fLineSpacing);
		StyleConstants.setSpaceAbove(attributes,fSpaceAbove);
		StyleConstants.setSpaceBelow(attributes,fSpaceBelow);

		return attributes;
	}
	private Color getInvertColor(Color color)
	{
		if(color == null)
			return Color.white;
		int red = 255 - color.getRed();
		int green = 255 - color.getGreen();
		int blue = 255 - color.getBlue();
		
		return new Color(red,green,blue);
	}
	private JFrame getFrame()
	{
		Object f = this;
		while( !(f instanceof JFrame) )
			f = ((Container)f).getParent();

		return (JFrame)f;
	}
}