/*
 * @(#)PDFDataServlet.java	Version 1.0  <April 02, 2003>
 *
 * Copyright 2000-2002 Elegant MicroWeb Technologies Pvt. Ltd. (EMTPL). All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * - Redistributions of source code is not permitted under any conditions.
 * 
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of Elegant MicroWeb Technologies Pvt. Ltd. or the names of contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. EMTPL AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES OR LIABILITIES
 * SUFFERED BY LICENSEE AS A RESULT OF  OR RELATING TO USE, MODIFICATION
 * OR DISTRIBUTION OF THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL
 * EMTPL OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE
 * DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,
 * ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF EMTPL HAS
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 * You acknowledge that Software is not designed, licensed or intended
 * for use in the design, construction, operation or maintenance of any
 * nuclear facility or any other illegal activity.
 */

import java.awt.*;
import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.elegantj.pdf.*;

public class PDFDataServlet extends HttpServlet
{
	private static final Color BACK_COLOR = new Color(223, 255, 255);
	private static final Color FORE_COLOR = new Color(0, 0, 117);

	public void service(HttpServletRequest request, HttpServletResponse response) throws ServletException
	{
		ServletOutputStream out = null;
		PDF pdf = null;
		MediaBox mediaBox = MediaBox.MEDIA_a4;
		Page page = null;
		PDFText pdfText = null;
		PDFImage pdfImage = null;
		PDFTable pdfTable = null;
		Vector data = null;

		try
		{
			response.setContentType("application/pdf");
			out = response.getOutputStream();

			// creating new pdf object, base class of pdf generator
			pdf = new PDF(Catalog.USEOUTLINES);

			// creates the PDFPage
			page = new Page(pdf.getPages(), mediaBox);

			pdfText = new PDFText("This PDF is dynamically generated and \nDisplays the detail of personal information.", 50, 25, mediaBox.width - 100, 80);
			pdfText.setFontSize(16);
			pdfText.setFillColor(FORE_COLOR);
			pdfText.setUnderLine(true);
			page.addText(pdfText);

			// creates the PDFTable object with x, y, width and height
			pdfTable = new PDFTable(50, mediaBox.y + 80, mediaBox.width - 100, 6 * 20);
			pdfTable.setDefaultBackground(BACK_COLOR);
			pdfTable.setDefaultForeground(FORE_COLOR);
			// get data from get data method
			data = getData(request);
			// sets the data for the PDFTable
			pdfTable.setData(data);
			// sets the row background at the 0th row
			pdfTable.setRowBackgroundAt(0, FORE_COLOR);
			pdfTable.setRowForegroundAt(0, BACK_COLOR);
			pdfTable.setColumnWidthAt(0, 90);
			// adds the PDFTable to the page
			page.addTable(pdfTable);

			pdfImage = new PDFImage(getClass().getResource("image.jpg"));
			pdfImage.setImageX(50);
			pdfImage.setImageY(220);
			pdfImage.setImageWidth(mediaBox.width - 100);
			pdfImage.setImageHeight(200);
			
			page.addImage(pdfImage);

			pdf.write(out);
			out.flush();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Returns the 2 dimensional data vector
	 * after creating from request
	 */
	public Vector getData(HttpServletRequest request)
	{
		String tempS = "";
		Vector data = new Vector();
		Vector temp = null;

		temp = new Vector();
		temp.addElement("Info Key");
		temp.addElement("Info Value");
		data.addElement(temp);

		temp = new Vector();
		temp.addElement("First Name");
		tempS = request.getParameter("t_customer_fname");
		if(tempS == null || tempS.trim().equals(""))
			tempS = "Undefined";
		temp.addElement(tempS);
		data.addElement(temp);

		temp = new Vector();
		temp.addElement("Last Name");
		tempS = request.getParameter("t_customer_lname");
		if(tempS == null || tempS.trim().equals(""))
			tempS = "Undefined";
		temp.addElement(tempS);
		data.addElement(temp);

		temp = new Vector();
		temp.addElement("Designation");
		tempS = request.getParameter("t_customer_designation");
		if(tempS == null || tempS.trim().equals(""))
			tempS = "Undefined";
		temp.addElement(tempS);
		data.addElement(temp);

		temp = new Vector();
		temp.addElement("Company Name");
		tempS = request.getParameter("t_company_name");
		if(tempS == null || tempS.trim().equals(""))
			tempS = "Undefined";
		temp.addElement(tempS);
		data.addElement(temp);

		temp = new Vector();
		temp.addElement("Phone No");
		tempS = request.getParameter("t_customer_phone_r");
		if(tempS == null || tempS.trim().equals(""))
			tempS = "Undefined";
		temp.addElement(tempS);
		data.addElement(temp);

		return data;
	}
}